import argparse
import os

import numpy as np

TYPE_MAPPER = {
    "Float32": np.float32,
    "Float16": np.float16,
    "Uint8": np.uint8,
}


class BinLoader:
    def __init__(self, bin_file):
        self.bin_file = bin_file
        if not os.path.exists(bin_file):
            raise Exception("File not exit: " + bin_file)
        self.bin_data = None

    def load(self):
        def __parse_bin_name(bin_file):
            filename = os.path.basename(bin_file)
            filename = os.path.splitext(filename)[0]
            infos = filename.split('_')
            # ops_info = infos[0]  # Conv2d--gradConv2D--Conv2DBackpropFilter-op865
            # ops_direct = infos[1]  # output
            # ops_idx = infos[2]  # 0
            ops_shape = infos[4:-2]  # [32, 4, 16, 16]
            ops_dtype = infos[-2]  # Float32
            return tuple(int(i) for i in ops_shape), TYPE_MAPPER[ops_dtype]

        bin_shape, bin_dtype = __parse_bin_name(self.bin_file)
        print(">>> parse bin name ok ~")
        print(">>> -- bin_shape = {}".format(bin_shape))
        print(">>> -- bin_dtype = {}".format(bin_dtype))

        self.bin_data = np.fromfile(self.bin_file, bin_dtype).reshape(bin_shape)
        print(">>> load bin to numpy success~")
        return self

    def is_nan_exist(self):
        if self.bin_data is None:
            print("Please call load() first!!!")
            return

        def __is_nan_exist(bin_data):
            if len(bin_data.shape) > 1:
                for item in bin_data:
                    if __is_nan_exist(item):
                        return True
            if np.isnan(bin_data).any():
                print(">>>>>>>> is nan", bin_data)
                return True
            return False

        return __is_nan_exist(self.bin_data)

    def print_bin(self):
        def __print_numpy_array(numpy_array, width=16):
            if len(numpy_array.shape) > 1:
                for item in numpy_array:
                    __print_numpy_array(item)

            height = int(len(numpy_array) / width) + 1
            for i in range(height):
                star_idx = i * width
                end_idx = (star_idx + width) if (star_idx + width) < len(numpy_array) else len(numpy_array)
                tmp_data = numpy_array[star_idx: end_idx]
                print("[{}:{}] -> {}".format(star_idx, end_idx, tmp_data))

        __print_numpy_array(self.bin_data)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-b', '--bin-file', type=str, default="./my.bin", help="bin file")

    args, _ = parser.parse_known_args()

    loader = BinLoader("")
    loader.load()
    loader.is_nan_exist()
    loader.print_bin()
    loader.print_bin()




from multiprocessing import Process, Manager
import numpy as np
import os
import inspect


class MutiProcessor():
    def __init__(self, np_data, h_num=1, w_num=1, cube_func=None):
        if len(np_data.shape) < 2:
            raise Exception("np_data must at least have dim 2.")
        self.np_data = np_data

        self.h_size, self.w_size = np_data.shape[:2]
        print(">>>>>>>-- self.h_size", self.h_size, ", self.w_size=", self.w_size)

        if h_num > self.h_size or w_num > self.w_size:
            raise Exception("h_num or w_num must at less then np_data shape {}. but got {}".format(np_data.shape[:2], (h_num, w_num)))
        self.h_num, self.w_num = h_num, w_num
        print(">>>>>>>-- self.h_num", self.h_num, ", self.w_num=", self.w_num)

        self.h_step, self.w_step = self.h_size // self.h_num, self.w_size // self.w_num
        print(">>>>>>>-- self.h_step", self.h_step, ", self.w_step=", self.w_step)

        self.__cube_func = cube_func
        self.__ret_dict = Manager().dict()

    def __get_location(self, i, j):
        start_h, end_h = i * self.h_step, (i + 1) * self.h_step
        start_w, end_w = j * self.w_step, (j + 1) * self.w_step
        end_h = end_h if end_h < self.h_size else self.h_size
        end_w = end_w if end_w < self.w_size else self.w_size
        return (start_h, start_w, end_h, end_w)

    def __cube_process(self, i, j):
        start_h, start_w, end_h, end_w = self.__get_location(i, j)
        cube_input = self.np_data[start_h:end_h, start_w:end_w]
        print(">>>>>>>>>__cube_process: ({},{}) = {} start...".format(i, j, cube_input.shape))

        cube_output = None
        if self.__cube_func:
            func_args = inspect.getfullargspec(self.__cube_func).args
            if len(func_args) == 1:
                cube_output = self.__cube_func(cube_input)
            elif len(func_args) == 2:
                cube_output = self.__cube_func(cube_input, (start_h, start_w, end_h, end_w))
            else:
                raise Exception("args num invalid." + str(func_args))
        print(">>>>>>>>>__cube_process: ({},{}) = {} end.".format(i, j, cube_output.shape if cube_output is not None else None))

        key = "_".join([str(x) for x in (start_h, start_w, end_h, end_w)])
        self.__ret_dict[key] = cube_output

    def process(self):
        _process_list = []
        for i in range(self.h_num):
            for j in range(self.w_num):
                p = Process(target=self.__cube_process, args=(i, j))
                p.start()
                _process_list.append(p)

        for p in _process_list:
            p.join()

        return self.__ret_dict


if __name__ == 'main':
    img_np = np.arange(10000).reshape(100, 100)
    def cube_func(cube_input, locs):
        pass
    processor = MutiProcessor(img_np, h_num=5, w_num=5, cube_func=gray2RGB)
    ret = processor.process()


def safe2int(val, def_val=None):
    try:
        return int(val)
    except ValueError:
        return val if def_val is None else def_val


def safe2float(val, def_val=None):
    try:
        return float(val)
    except ValueError:
        return val if def_val is None else def_val



import json
import os

DEVICE_NUM = 8
HCCN_CONF_FILE = '/etc/hccn.conf'
BOARD_IDS = {
    'A+K': '0x002f',
    'A+X': '0x0000'
}
DEVICE_TYPE = 'A+K'

SERVER_ID = '10.10.10.10'


def gen_rank_table_file(server_id=SERVER_ID, device_type=DEVICE_TYPE, visible_devices=None,
                        hccn_conf_file=None):
    server_id = SERVER_ID if server_id is None else server_id
    device_type = DEVICE_TYPE if device_type is None else device_type
    visible_devices = [str(x) for x in range(DEVICE_NUM)] if visible_devices is None else visible_devices
    hccn_conf_file = HCCN_CONF_FILE if hccn_conf_file is None else hccn_conf_file

    device_num = len(visible_devices)

    def get_device_ips(cfg_file):
        rst = {}
        with open(cfg_file, 'r') as f:
            for line in f.readlines():
                if not line.startswith('address_'):
                    continue
                device_id, device_ip = line.strip().split('=')
                device_id = device_id.split('_')[1]
                rst[device_id] = device_ip
                print('device_id:{}, device_ip:{}'.format(device_id, device_ip))
        return rst

    if os.path.exists(hccn_conf_file):
        device_ips = get_device_ips(hccn_conf_file)
    else:
        print("Warning: Please specify the param `hccn_conf_file` so that we can get the right device ip.")
        device_ips = {
            '0': '192.168.1.101',
            '1': '192.168.2.101',
        }
        device_num = len(device_ips)
        visible_devices = [str(x) for x in range(device_num)]

    # rank_table template
    rank_table = {
        'board_id': BOARD_IDS.get(device_type),
        'chip_info': '910',
        'deploy_mode': 'lab',
        'group_count': '1',
        'group_list': [],  # set later
        'para_plane_nic_location': 'device',
        'para_plane_nic_num': '',
        'para_plane_nic_name': [],  # set later
        'status': 'completed'
    }
    group_list = [
        {
            'device_num': str(device_num),
            'server_num': '1',
            'group_name': '',
            'instance_count': str(device_num),
            'instance_list': [],  # set later
        }
    ]
    rank_table['group_list'] = group_list

    instance_list = []
    for i in range(device_num):
        instance = {
            'rank_id': str(i),
            'server_id': server_id,
            'devices': {
                'device_id': visible_devices[i],
                'device_ip': device_ips[visible_devices[i]],
            }
        }
        instance_list.append(instance)
    group_list[0]['instance_list'] = instance_list

    para_plane_nic_name = []
    for i in range(device_num):
        eth_id = visible_devices[i]
        para_plane_nic_name.append('eth{}'.format(eth_id))
    rank_table['para_plane_nic_name'] = para_plane_nic_name

    usable_dev = ''.join(visible_devices)
    table_fn = os.path.join(os.getcwd(), 'rank_table_{}p_{}_{}.json'.format(device_num, usable_dev, server_id))
    with open(table_fn, 'w') as table_fp:
        json.dump(rank_table, table_fp, indent=4)
    return table_fn


if __name__ == '__main__':
    gen_rank_table_file()



def create_net():
    import mindspore.common.dtype as mstype
    from mindspore import nn
    from mindspore.common.initializer import initializer

    def _make_layer(base, batch_norm):
        layers = []
        in_channels = 3
        for v in base:
            if v == 'M':
                layers += [nn.MaxPool2d(kernel_size=2, stride=2)]
            else:
                weight_shape = (v, in_channels, 3, 3)
                weight = initializer('XavierUniform', shape=weight_shape, dtype=mstype.float32)
                conv2d = nn.Conv2d(in_channels=in_channels,
                                   out_channels=v,
                                   kernel_size=3,
                                   padding=0,
                                   pad_mode="same",
                                   has_bias=False,
                                   weight_init=weight)
                if batch_norm:
                    layers += [conv2d, nn.BatchNorm2d(v), nn.ReLU()]
                else:
                    layers += [conv2d, nn.ReLU()]
                in_channels = v
        return nn.SequentialCell(layers)

    class Vgg(nn.Cell):
        def __init__(self, base, num_classes=1000, batch_norm=False, has_dropout=True, phase="train",
                     include_top=True):
            super(Vgg, self).__init__()
            self.layers = _make_layer(base, batch_norm=batch_norm)
            self.include_top = include_top
            self.flatten = nn.Flatten()
            dropout_ratio = 0.5
            if not has_dropout or phase == "test":
                dropout_ratio = 1.0
            self.classifier = nn.SequentialCell([
                nn.Dense(512 * 7 * 7, 4096),
                nn.ReLU(),
                nn.Dropout(dropout_ratio),
                nn.Dense(4096, 4096),
                nn.ReLU(),
                nn.Dropout(dropout_ratio),
                nn.Dense(4096, num_classes),
            ])

        def construct(self, x):
            x = self.layers(x)
            if self.include_top:
                x = self.flatten(x)
                x = self.classifier(x)
            return x

    cfg = {
        '11': [64, 'M', 128, 'M', 256, 256, 'M', 512, 512, 'M', 512, 512, 'M'],
        '13': [64, 64, 'M', 128, 128, 'M', 256, 256, 'M', 512, 512, 'M', 512, 512, 'M'],
        '16': [64, 64, 'M', 128, 128, 'M', 256, 256, 256, 'M', 512, 512, 512, 'M', 512, 512, 512, 'M'],
        '19': [64, 64, 'M', 128, 128, 'M', 256, 256, 256, 256, 'M', 512, 512, 512, 512, 'M', 512, 512, 512, 512, 'M'],
    }

    return Vgg(cfg['16'], num_classes=10)


if __name__ == '__main__':
    net = create_net()
    print(net)



import logging
import os
import sys


LOG_LEVEL_MAP={
    "DEBUG": logging.DEBUG,
    "INFO": logging.INFO,
    "WARNING": logging.INFO,
    "ERROR": logging.ERROR,
    "CRITICAL": logging.CRITICAL
}


def get_logger(level="INFO"):
    # 
    _level = LOG_LEVEL_MAP.get(level.upper(), logging.INFO)
    _formatter = logging.Formatter("%(asctime)s [%(levelname)-7s] [%(process)d] [%(module)s.%(funcName)s():%(lineno)-4d] - %(message)s")

    logger = logging.getLogger()
    logger.setLevel(_level)

    log_file_handler = logging.FileHandler("log.txt")
    log_file_handler.setLevel(_level)
    log_file_handler.setFormatter(_formatter)

    log_console_handler = logging.StreamHandler()
    log_console_handler.setLevel(_level)
    log_console_handler.setFormatter(_formatter)

    logger.addHandler(log_file_handler)
    logger.addHandler(log_console_handler)

    return logger


if __name__ == '__main__':
    logger = get_logger(level="DEBUG")
    logger.debug("this is debug msg")
    logger.info("test param is: %s", "haha")
    logger.warning("this is warning msg")
    logger.error("this is error msg")



from erhsh.common.checkpoint import CheckpointLoader


class MsCkptLoader(CheckpointLoader):
    def _load_checkpoint(self):
        from mindspore.train.serialization import load_checkpoint
        param_dict = load_checkpoint(self.checkpoint_path)
        return {k: v.asnumpy() for k, v in param_dict.items()}



import torch


def test_simple_grad():
    # inputs
    input_x = torch.tensor([1, -1, 0, 2, -5], dtype=torch.float32, requires_grad=True)
    print("input_x:", type(input_x), input_x.detach().numpy())

    # backward
    ones = torch.ones(size=(5,))
    input_x.backward(gradient=ones)
    grad = input_x.grad
    print("input_x.grad:", type(grad), grad.numpy())


if __name__ == '__main__':
    test_simple_grad()



import argparse
import erhsh.utils as eut


class ArgumentParser(argparse.ArgumentParser):
    def __init__(self, *args, **kwargs):
        super(ArgumentParser, self).__init__(*args, **kwargs)
        self.add_argument("--device_target", type=str, default="CPU", choices=['CPU', 'Ascend', 'GPU'],
                          help="device target. (Default: CPU)")
        self.add_argument("--graph_mode", action="store_true", help="graph mode flag. (Default: False)")
        self.add_argument("--save_graphs", action="store_true", help="save graphs flag. (Default: False)")
        self.add_argument("--visible_devices", type=str, help="num0,num1,num2...")
        self.add_argument("--do_eval", action="store_true", help="do eval flag. (Default: False)")
        self.add_argument("--ckpt_path", type=str, default="./ckpt", help="save or eval ckpt path. (Default: ./ckpt)")
        self.add_argument("--epoch_size", type=int, default=1, help="epoch size. (Default: 1)")
        self.add_argument("--no_sink", action="store_false", help="sink flag. (Default: True)")
        self.add_argument("--sink_size", type=int, default=-1, help="sink size. (Default: -1)")
        self.add_argument("--data_path", type=str, default="./data", help="data path. (Default: ./data)")
        self.add_argument("--batch_size", type=int, default=32, help="batch size. (Default: 32)")

        self._args = None
        self._unparsed = None

    def print_args(self):
        tbl = eut.TblPrinter("Param Key", "Param Value")
        tbl.add_row("--device_target", str(self.device_target))
        tbl.add_row("--graph_mode", str(self.graph_mode))
        tbl.add_row("--save_graphs", str(self.save_graphs))
        tbl.add_row("--visible_devices", str(self.visible_devices))
        tbl.add_row("--do_eval", str(self.do_eval))
        tbl.add_row("--ckpt_path", str(self.ckpt_path))
        tbl.add_row("--epoch_size", str(self.epoch_size))
        tbl.add_row("--do_sink", str(self.do_sink))
        tbl.add_row("--sink_size", str(self.sink_size))
        tbl.add_row("--data_path", str(self.data_path))
        tbl.add_row("--batch_size", str(self.batch_size))
        tbl.print()

    def parse_known_args(self, args_=None, namespace=None):
        if self._args:
            return self._args, self._unparsed

        self._args, self._unparsed = super(ArgumentParser, self).parse_known_args(args=args_, namespace=namespace)
        return self._args, self._unparsed

    # device_target
    @property
    def device_target(self):
        return self._args.device_target

    @device_target.setter
    def device_target(self, val):
        self._args.device_target = val

    # graph_mode
    @property
    def graph_mode(self):
        return self._args.graph_mode

    @graph_mode.setter
    def graph_mode(self, val):
        self._args.graph_mode = val

    # save_graphs
    @property
    def save_graphs(self):
        return self._args.save_graphs

    @save_graphs.setter
    def save_graphs(self, val):
        self._args.save_graphs = val

    # visible_devices
    @property
    def visible_devices(self):
        return self._args.visible_devices

    @visible_devices.setter
    def visible_devices(self, val):
        self._args.visible_devices = val

    # do_eval
    @property
    def do_eval(self):
        return self._args.do_eval

    @do_eval.setter
    def do_eval(self, val):
        self._args.do_eval = val

    # ckpt_path
    @property
    def ckpt_path(self):
        return self._args.ckpt_path

    @ckpt_path.setter
    def ckpt_path(self, val):
        self._args.ckpt_path = val

    # epoch_size
    @property
    def epoch_size(self):
        return self._args.epoch_size

    @epoch_size.setter
    def epoch_size(self, val):
        self._args.epoch_size = val

    # do_sink
    @property
    def do_sink(self):
        return self._args.no_sink

    @do_sink.setter
    def do_sink(self, val):
        self._args.no_sink = val

    # sink_size
    @property
    def sink_size(self):
        return self._args.sink_size

    @sink_size.setter
    def sink_size(self, val):
        self._args.sink_size = val

    # data_path
    @property
    def data_path(self):
        return self._args.data_path

    @data_path.setter
    def data_path(self, val):
        self._args.data_path = val

    # batch_size
    @property
    def batch_size(self):
        return self._args.batch_size

    @batch_size.setter
    def batch_size(self, val):
        self._args.batch_size = val

    def __getattr__(self, item):
        if hasattr(self._args, item):
            return getattr(self._args, item)
        return None

    def __repr__(self):
        return str(self._args)

    def __str__(self):
        return str(self._args)


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument("--data_path", type=str, help="dataset path")
    parser.add_argument("--batch_size", type=int, default=32, help="batch size")

    args = parser.parse_known_args()
    args.data_path = "/my/data/path"
    args.epoch_size = 123
    args.do_sink = False

    print(type(args))
    print(args)
    print(args.data_path)
    print(args.epoch_size)
    print(args.do_sink)



import os
import numpy as np
import cv2
from PIL import Image, ImageFile


ImageFile.LOAD_TRUNCATED_IMAGES=True
Image.MAX_IMAGE_PIXELS=None


def __cal_hist(a, b, n):
    # a -> label_true, b -> label_predict
    k = (a >= 0) & (a < n)
    return np.bincount(n * a[k].astype(np.int32) + b[k], minlength=n ** 2).reshape(n, n)

def calc_hist(label_img, predict_img, num_classes):
    label_np = label_img
    predict_np = predict_img

    if isinstance(label_img, str):
        label_np = np.array(Image.open(label_img).convert('L'), dtype=np.uint8)
    if isinstance(predict_img, str):
        predict_np = np.array(Image.open(predict_img).convert('L'), dtype=np.uint8)

    print(">>>label_np shape:", label_np.shape)
    print(">>>predict_np shape:", predict_np.shape)

    return __cal_hist(label_np.flatten(), predict_np.flatten(), num_classes)


def calc_mean_iou(label_imgs, predict_imgs, num_classes):
    hist = np.zeros((num_classes, num_classes))
    for lable_img, predict_img in zip(label_imgs, predict_imgs):
        hist += calc_hist(lable_img, predict_img, num_classes)

    print(hist)
    iu = np.diag(hist) / (hist.sum(1) + hist.sum(0) - np.diag(hist))
    mean_iou = np.nanmean(iu)
    print('per-class IoU', iu)
    print('mean IoU', mean_iou)
    return mean_iou


if __name__ == '__main__':
    names = """
        GF2_PMS1__L1A0000647768-MSS1
        GF2_PMS1__L1A0001094941-MSS1
        GF2_PMS1__L1A0001169100-MSS1
        GF2_PMS1__L1A0001491417-MSS1
        GF2_PMS1__L1A0001537716-MSS1
        GF2_PMS1__L1A0001553848-MSS1
        GF2_PMS1__L1A0001562063-MSS1
        GF2_PMS1__L1A0001734328-MSS1
        GF2_PMS2__L1A0000564692-MSS2
        GF2_PMS2__L1A0000647892-MSS2
        GF2_PMS2__L1A0000718813-MSS2
        GF2_PMS2__L1A0000948183-MSS2
        GF2_PMS2__L1A0001015596-MSS2
        GF2_PMS2__L1A0001028977-MSS2
        GF2_PMS2__L1A0001035795-MSS2
        GF2_PMS2__L1A0001119060-MSS2
        GF2_PMS2__L1A0001336975-MSS2
        GF2_PMS2__L1A0001378491-MSS2
        GF2_PMS2__L1A0001389317-MSS2
        GF2_PMS2__L1A0001416140-MSS2
        GF2_PMS2__L1A0001457157-MSS2
        GF2_PMS2__L1A0001537637-MSS2
        GF2_PMS2__L1A0001566653-MSS2
        GF2_PMS2__L1A0001577567-MSS2
        GF2_PMS2__L1A0001633212-MSS2
        GF2_PMS2__L1A0001708232-MSS2
        GF2_PMS2__L1A0001757317-MSS2
        GF2_PMS2__L1A0001757484-MSS2
        GF2_PMS2__L1A0001787080-MSS2
        GF2_PMS2__L1A0001886305-MSS2
    """

    names = names.split()
    base_dir = "/home/cj/pictmp.gid_gray_comp/"
    num_classes = 6

    lbl_paths = [os.path.join(base_dir, "{}_lbl_rank{}.tif".format(x, "0~32")) for x in names]
    img_paths = [os.path.join(base_dir, "{}_rank{}.tif".format(x, "0~32")) for x in names]

    calc_mean_iou(lbl_paths, img_paths, num_classes)


import os

import erhsh.ms as ems


def create_dataset(batch_size=32):
    # import
    import mindspore.dataset.engine as de
    import numpy as np
    from mindspore.common import set_seed

    set_seed(1)

    # shard
    num_shards = shard_id = None
    rand_size = os.getenv("RANK_SIZE")
    rand_id = os.getenv("RANK_ID")
    if rand_size is not None and rand_id is not None:
        num_shards = int(rand_size)
        shard_id = int(rand_id)

    # define dataset
    class BaseDataset:
        def __init__(self):
            self.samples = []
            self._load_samples()

        def __getitem__(self, index):
            sample = self.samples[index]
            return sample[0], sample[1]

        def _load_samples(self):
            self.samples.append(
                [np.random.rand(3, 4, 5).astype(np.float32),
                 np.random.randint(10, size=()).astype(np.int32)])

        def __len__(self):
            return len(self.samples)

    # define dataset
    ds = de.GeneratorDataset(source=BaseDataset(), column_names=['image', 'label'],
                             num_shards=num_shards, shard_id=shard_id)

    # map ops
    ds = ds.map(input_columns=["image"], operations=lambda img: img, num_parallel_workers=8)

    # batch & repeat
    ds = ds.batch(batch_size=batch_size, drop_remainder=False)
    ds = ds.repeat(count=1)

    return ds


if __name__ == '__main__':
    dataset = create_dataset()
    ems.print_ds_info(dataset)
    ems.print_ds_data(dataset)



from erhsh.utils import TblPrinter


class JupyterHelper:
    def __init__(self):
        self.cmd_items = list()

    def add(self, *cmd_item):
        self.cmd_items.append(cmd_item)

    def show(self):
        tp = TblPrinter("Name", "Cmd Example")
        for cmd_item in self.cmd_items:
            tp.add_row(*cmd_item)
        tp.print()


jupyter_helper = JupyterHelper()

jupyter_helper.add("*install", '''
pip install jupyter
'''.strip())
jupyter_helper.add("*generate config", '''
jupyter notebook --generate-config
'''.strip())
jupyter_helper.add("*password", '''
jupyter notebook password
'''.strip())
jupyter_helper.add("edit config", '''
c.NotebookApp.ip="*"; c.NotebookApp.open_browser=False; c.NotebookApp.notebook_dir="/home/workspace/";
'''.strip())
jupyter_helper.add("contrib nbextensions", '''
pip install jupyter_contrib_nbextensions && jupyter contrib nbextension install --user
'''.strip())
jupyter_helper.add("nbextensions", '''
pip install jupyter_nbextensions_configurator && jupyter nbextensions_configurator enable --user
'''.strip())
jupyter_helper.add("*start", '''
jupyter notebook --notebook-dir='./' --ip='*' --port=8888 --allow-root --no-browser > out.log 2>&1 &
'''.strip())



def create_net():
    from mindspore import nn
    from mindspore.common.initializer import Normal

    class LeNet5(nn.Cell):
        def __init__(self, num_class=10, num_channel=1):
            super(LeNet5, self).__init__()
            self.conv1 = nn.Conv2d(num_channel, 6, 5, pad_mode='valid')
            self.conv2 = nn.Conv2d(6, 16, 5, pad_mode='valid')
            self.fc1 = nn.Dense(16 * 5 * 5, 120, weight_init=Normal(0.02))
            self.fc2 = nn.Dense(120, 84, weight_init=Normal(0.02))
            self.fc3 = nn.Dense(84, num_class, weight_init=Normal(0.02))
            self.relu = nn.ReLU()
            self.max_pool2d = nn.MaxPool2d(kernel_size=2, stride=2)
            self.flatten = nn.Flatten()

        def construct(self, x):
            x = self.max_pool2d(self.relu(self.conv1(x)))
            x = self.max_pool2d(self.relu(self.conv2(x)))
            x = self.flatten(x)
            x = self.relu(self.fc1(x))
            x = self.relu(self.fc2(x))
            x = self.fc3(x)
            return x

    return LeNet5()


if __name__ == '__main__':
    net = create_net()
    print(net)



from erhsh.utils import TblPrinter


class VimHelper:
    def __init__(self):
        self.cmd_items = list()

    def add(self, *cmd_item):
        self.cmd_items.append(cmd_item)

    def show(self):
        tp = TblPrinter("Name", "Cmd Example")
        for cmd_item in self.cmd_items:
            tp.add_row(*cmd_item)
        tp.print()


vim_helper = VimHelper()

vim_helper.add('show unicode', ':digraphs')
vim_helper.add('visualize chars', '1. :set list     2. :set listchars=eol:$,tab:>-,trail:~,extends:>,precedes:<')
vim_helper.add('tab2space', '1. :set tabstop=4  2. :set shiftwidth=4    3. :set expandtab')



import os

import erhsh.ms as ems


def create_dataset(data_path, is_train=True, batch_size=32):
    # import
    import mindspore.dataset.engine as de
    import mindspore.dataset.vision.c_transforms as C
    from mindspore.common import set_seed

    set_seed(1)

    # shard
    num_shards = shard_id = None
    rand_size = os.getenv("RANK_SIZE")
    rand_id = os.getenv("RANK_ID")
    if rand_size is not None and rand_id is not None:
        num_shards = int(rand_size)
        shard_id = int(rand_id)

    # define dataset
    ds = de.MindDataset(data_path, columns_list=['data'], shuffle=True,
                        num_shards=num_shards, shard_id=shard_id,
                        num_parallel_workers=8, num_samples=None)

    # map ops
    ds = ds.map(input_columns=["data"], operations=C.Decode())
    ds = ds.map(input_columns=["data"], operations=C.Normalize(mean=[0.485 * 255, 0.456 * 255, 0.406 * 255],
                                                               std=[0.229 * 255, 0.224 * 255, 0.225 * 255]))
    ds = ds.map(input_columns=["data"], operations=C.Resize((224, 224)))
    ds = ds.map(input_columns=["data"], operations=C.HWC2CHW())

    # batch & repeat
    ds = ds.batch(batch_size=batch_size, drop_remainder=is_train)
    ds = ds.repeat(count=1)

    return ds


if __name__ == '__main__':
    dataset = create_dataset("D:/Data/dataset/voc2012/t1/t1.m0", is_train=True)
    ems.print_ds_info(dataset)
    ems.print_ds_data(dataset)



from .ds_mocker import DatasetMocker
from .ds_printer import print_ds_info, print_ds_performance, print_ds_data
from .ds_wrapper import DatasetWrapper, ImageDatasetWrapper

__all__ = [
    "print_ds_info",
    "print_ds_performance",
    "print_ds_data",
    "DatasetMocker",
    "DatasetWrapper",
    "ImageDatasetWrapper",
]



import erhsh.ms as ems


def main():
    ems.Trainer(name="simplenet") \
        .set_dataset(ems.create_mocker_dataset()) \
        .set_network(ems.SimpleNet()) \
        .set_callbacks([ems.TimeMonitor()]) \
        .run()


if __name__ == '__main__':
    main()



import os

import erhsh.ms as ems


def create_dataset(data_path, is_train=True, batch_size=32):
    # import
    import mindspore.common.dtype as mstype
    import mindspore.dataset.engine as de
    import mindspore.dataset.transforms.c_transforms as C2
    import mindspore.dataset.vision.c_transforms as C
    from mindspore.common import set_seed

    set_seed(1)

    # shard
    num_shards = shard_id = None
    rand_size = os.getenv("RANK_SIZE")
    rand_id = os.getenv("RANK_ID")
    if rand_size is not None and rand_id is not None:
        num_shards = int(rand_size)
        shard_id = int(rand_id)

    # define dataset
    data_path = os.path.join(data_path, "cifar-10-batches-bin" if is_train else "cifar-10-verify-bin")
    ds = de.Cifar10Dataset(data_path, shuffle=True, num_shards=num_shards, shard_id=shard_id,
                           num_parallel_workers=8, num_samples=None)

    # define ops
    comps_ops = list()

    # train or val
    if is_train:
        comps_ops.append(C.RandomCrop((32, 32), (4, 4, 4, 4)))
        comps_ops.append(C.RandomHorizontalFlip(prob=0.5))

    comps_ops.append(C.Resize((224, 224)))
    comps_ops.append(C.Rescale(1 / 255.0, 0.))
    comps_ops.append(C.Normalize(mean=[0.4914, 0.4822, 0.4465], std=[0.2023, 0.1994, 0.2010]))
    comps_ops.append(C.HWC2CHW())

    # map ops
    ds = ds.map(input_columns=["image"], operations=comps_ops, num_parallel_workers=8)
    ds = ds.map(input_columns=["label"], operations=C2.TypeCast(mstype.int32), num_parallel_workers=8)

    # batch & repeat
    ds = ds.batch(batch_size=batch_size, drop_remainder=is_train)
    ds = ds.repeat(count=1)

    return ds


if __name__ == '__main__':
    dataset = create_dataset("D:/Data/dataset/cifar-10-binary", is_train=True)
    ems.print_ds_info(dataset)
    ems.print_ds_data(dataset)



import os

import click

import erhsh
from erhsh.ms.tools.ckpt import MsCkptLoader
from erhsh.ms.tools.bin_tool import BinLoader
from erhsh.ms.tools.hccl_tool import gen_rank_table_file
from erhsh.ms.tools.hccl_tool_v1 import gen_rank_table_file as gen_rank_table_file_v1


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo("Supported MindSpore version is %s" % erhsh.ms.version.__version__)
    ctx.exit()


@click.group(help='This is MindSpore Command Line Console.')
@click.option("--version", is_flag=True, callback=print_version, expose_value=False,
              help="Show Supported MindSpore version.")
def ms_cli():
    pass


@ms_cli.command("hccl", help="Generate HCCL Rank Table File")
@click.option("--server_id", "-sid", prompt="Server Id(IP)", help="server id")
@click.option("--device_type", "-dt", prompt="Device Type", help="device type")
@click.option("--visible_devices", "-vds", help="visible devices")
@click.option("--hccn_conf_file", "-hcf", help="hccn config file")
def gen_hccl_rank_table(server_id, device_type, visible_devices, hccn_conf_file):
    gen_file_path = gen_rank_table_file(server_id=server_id, device_type=device_type, visible_devices=visible_devices,
                                        hccn_conf_file=hccn_conf_file)
    print("Generate success:", gen_file_path)


@ms_cli.command("hcclv1", help="Generate HCCL Rank Table File v1.0")
@click.option("--server_id", "-sid", prompt="Server Id(IP)",
              default=lambda: os.getenv("SSH_CONNECTION", "ip port x.x.x.x").split()[2], help="server id")
@click.option("--visible_devices", "-vds", prompt="Visible Devices",
              default="0,1,2,3,4,5,6,7",  help="visible devices")
@click.option("--hccn_conf_file", "-hcf", help="hccn config file")
def gen_hccl_rank_table_v1(server_id, visible_devices, hccn_conf_file):
    gen_file_path = gen_rank_table_file_v1(server_id=server_id, visible_devices=visible_devices,
                                           hccn_conf_file=hccn_conf_file)
    print("Generate success:", gen_file_path)


@ms_cli.command("ckpt", help="Get checkpoint information")
@click.option("--ckpt_path", "-p", required=True, help="checkpoint file path")
@click.option("--filter_key", "-f", default=None, help="key filter condition, regular supported")
@click.option("--key", "-k", default=None, help="specify key")
@click.option("--dump", "-d", is_flag=True, default=False, help="flag dump data")
@click.option("--dump_to", "-t", default="./", help="where dump to")
def ckpt(ckpt_path, filter_key, key, dump, dump_to):
    ms_loader = MsCkptLoader(ckpt_path)
    if key:
        if dump:
            ms_loader.get_dump(key, dump_to=dump_to)
        else:
            ms_loader.get(key)
    else:
        if dump:
            ms_loader.list_dump(filter_key=filter_key, dump_to=dump_to)
        else:
            ms_loader.list(filter_key=filter_key)


@ms_cli.command("bin", help="Get bin information")
@click.option("--bin_path", "-p", required=True, help="bin file path")
def bin(bin_path):
    bin_loader = BinLoader(bin_path).load()
    bin_loader.is_nan_exist()
    bin_loader.print_bin()


if __name__ == '__main__':
    ms_cli()



import os
from datetime import datetime


def list_sub_dirs(dir_path):
    if not os.path.exists(dir_path):
        raise RuntimeError("invalid dir path.")

    if not os.path.isdir(dir_path):
        raise RuntimeError("invalid dir path.")

    sub_dirs = []
    sub_files = os.listdir(dir_path)
    for sub_file_name in sub_files:
        sub_file_path = os.path.join(dir_path, sub_file_name)
        if not os.path.isdir(sub_file_path):
            continue
        sub_dirs.append(sub_file_path)
    return sub_dirs


def flatten_dir_path(dir_path):
    sub_dirs = list_sub_dirs(dir_path)
    for sub_dir_path in sub_dirs:
        sub_sub_dirs = list_sub_dirs(sub_dir_path)
        for sub_sub_dir_path in sub_sub_dirs:
            sub_sub_dir_name = os.path.basename(sub_sub_dir_path)
            target_dir_path = '_'.join([sub_dir_path, sub_sub_dir_name])
            print(f"move: {sub_sub_dir_path} -> {target_dir_path}")
            os.renames(sub_sub_dir_path, target_dir_path)


def create_dir(dir_path):
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)
        return dir_path
    return create_dir(dir_path + "." + datetime.now().strftime("%Y%m%d%H%M%S"))



import abc
import os
import re

import numpy as np

from erhsh.utils.print_util import TblPrinter


class CheckpointLoader(object):
    def __init__(self, checkpoint_path):
        self.checkpoint_path = checkpoint_path

    @abc.abstractmethod
    def _load_checkpoint(self):
        pass

    def __list(self, filter_key=None):
        param_dict = self._load_checkpoint()

        filter_dict = {}
        for k, v in param_dict.items():
            if filter_key \
                    and (filter_key not in k) \
                    and (not re.search(filter_key, k, re.M | re.I)):
                continue
            filter_dict[k] = v

        return param_dict, filter_dict

    def __get(self, key):
        param_dict = self._load_checkpoint()
        return param_dict.get(key)

    def list(self, filter_key=None):
        param_dict, filter_dict = self.__list(filter_key=filter_key)

        ret = {}
        tp = TblPrinter("Param Keys", "Value Shape")
        for k, v in filter_dict.items():
            v = str(v.shape)
            ret[k] = v
            tp.add_row(k, v)
        tp.print()

        print("Filter/Total: {}/{}".format(len(ret), len(param_dict)))
        return ret

    def get(self, key):
        v = self.__get(key)
        if v is None:
            print("param key not found! key={}".format(key))
            return

        tp = TblPrinter("Param Keys", "Value Shape", "Value Type")
        tp.add_row(key, str(v.shape), str(v.dtype))
        vf = v.flatten()
        length = len(vf)
        if length <= 100:
            tp.add_row(vf)
        elif length <= 200:
            tp.add_row(vf[:100])
            tp.add_row(vf[100:])
        else:
            tp.add_row(vf[:100])
            tp.add_row("...")
            tp.add_row(vf[-100:])
        tp.print()

        print("Max:{:.7f}, Min:{:.7f}, Mean:{:.7f}".format(v.max(), v.min(), v.mean()))

    def list_dump(self, filter_key=None, dump_to=None):
        print("Begin dump {0} to {1}".format(self.checkpoint_path, dump_to))
        print("Filter is: {}".format(filter_key))

        _, filter_dict = self.__list(filter_key=filter_key)

        for k, v in filter_dict.items():
            dump_file = os.path.join(dump_to, k + '.npy')
            np.save(dump_file, v)
            print("Dump to: {}".format(dump_file))

    def get_dump(self, key, dump_to=None):
        print("Begin dump {0} to {1}".format(self.checkpoint_path, dump_to))
        print("Key is: {}".format(key))

        v = self.__get(key)
        dump_file = os.path.join(dump_to, key + '.npy')
        np.save(dump_file, v)
        print("Dump to: {}".format(dump_file))



from .ds_00_mocker import create_dataset as create_mocker_dataset
from .ds_02_mind import create_dataset as create_mind_dataset
from .ds_03_imagefolder import create_dataset as create_imagefolder_dataset
from .ds_04_cifar10 import create_dataset as create_cifar10_dataset
from .ds_05_mnist import create_dataset as create_mnist_dataset

__all__ = [
    "create_mocker_dataset",
    "create_mind_dataset",
    "create_imagefolder_dataset",
    "create_cifar10_dataset",
    "create_mnist_dataset",
]



import numpy as np

import erhsh.ms as ems

np.random.seed(1)


def create_dataset():
    return ems.DatasetMocker().set_features({
        "data": lambda: np.random.rand(1, 32, 32).astype(np.float32),
        "label": lambda: np.random.randint(10, size=()).astype(np.int32)
    }).set_data_len(3 * 32).set_batch_size(32).mock()


if __name__ == '__main__':
    dataset = create_dataset()
    ems.print_ds_info(dataset)
    ems.print_ds_data(dataset)



import time

import numpy as np

np.set_printoptions(suppress=True)


def print_ds_info(ds, prefix="|-"):
    print("------ Dataset Information Is: ------", flush=True)
    print("{}get_dataset_size={}".format(prefix, ds.get_dataset_size()), flush=True)
    print("{}get_repeate_count={}".format(prefix, ds.get_repeat_count()), flush=True)
    print("{}output_types={}".format(prefix, ds.output_types()), flush=True)
    print("{}output_shapes={}".format(prefix, ds.output_shapes()), flush=True)

    for x in ds.create_dict_iterator(output_numpy=True):
        keys = x.keys()
        for k in keys:
            v = x.get(k)
            print("{} {} -> {} {}".format(prefix, k, v.shape, v.dtype), flush=True)
        break


def _print_cost(costs, decimals=None, prefix=""):
    print("{}AVG:{}".format(prefix, np.around(np.mean(costs), decimals=decimals)), flush=True)
    print("{}99%:{}".format(prefix, np.around(np.percentile(costs, 99), decimals=decimals)), flush=True)
    print("{}90%:{}".format(prefix, np.around(np.percentile(costs, 90), decimals=decimals)), flush=True)
    print("{}75%:{}".format(prefix, np.around(np.percentile(costs, 75), decimals=decimals)), flush=True)
    print("{}50%:{}".format(prefix, np.around(np.percentile(costs, 50), decimals=decimals)), flush=True)
    print("{}Max:{}".format(prefix, np.around(np.max(costs), decimals=decimals)), flush=True)
    print("{}Min:{}".format(prefix, np.around(np.min(costs), decimals=decimals)), flush=True)
    print("{}Sum:{}".format(prefix, np.around(np.sum(costs), decimals=decimals)), flush=True)


def print_ds_performance(ds, step1=10, step2=None, decimals=9, prefix="|-"):
    i = 0
    costs = []
    ss_time = s_time = time.time()
    for _ in ds.create_dict_iterator(output_numpy=True):
        i += 1
        e_time = time.time()
        c_time = round(e_time - s_time, decimals)
        s_time = e_time

        costs.append(c_time)

        if step2 and i % step2 == 0:
            print("cur index:", i, ", cur cost:", c_time, flush=True)
            _print_cost(costs, decimals=decimals, prefix=prefix)
        elif step1 and i % step1 == 0:
            print("cur index:", i, ", cur cost:", c_time, flush=True)

    print("Total Count: {}, total cost: {}".format(i, round(time.time() - ss_time, decimals)), flush=True)
    _print_cost(costs, decimals=decimals, prefix=prefix)

    print("Save numpy start...")
    np.savetxt("time_costs.txt", np.array(costs), fmt="%.{}f".format(decimals))
    print("Save numpy success~")


def _get_value(v):
    from mindspore import Tensor
    if isinstance(v, Tensor):
        v = v.asnumpy()

    return v


def print_ds_data(ds, prefix=">>>", row_limit=10, col_limit=100):
    i = 0
    for x in ds.create_dict_iterator(output_numpy=True):
        i += 1
        if i > row_limit:
            break
        print("-" * 32, "item:", i, "-" * 32, flush=True)
        keys = x.keys()
        for k in keys:
            v = x.get(k)
            vv = _get_value(v).flatten()
            if len(vv) > col_limit:
                limit = int(col_limit / 2)
                print("{} {}->{}\n{}\n...\n{}".format(prefix, k, v.shape, vv[:limit], vv[-limit:]), flush=True)
            else:
                print("{} {}->{}\n{}".format(prefix, k, v.shape, vv[:]), flush=True)



import click

import erhsh
from erhsh.tools.bash_helper import bash_helper
from erhsh.tools.git_helper import git_helper
from erhsh.tools.jupyter_helper import jupyter_helper
from erhsh.tools.vim_helper import vim_helper

from erhsh import utils as eut


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo("tools version is %s" % erhsh.tools.version.__version__)
    ctx.exit()


@click.group(help='This is Tools Command Line Console.')
@click.option("--version", is_flag=True, callback=print_version, expose_value=False,
              help="Show Tools Command Line Version.")
def tools_cli():
    pass


@tools_cli.command("git", help="Get Git Helper")
def git():
    git_helper.show()


@tools_cli.command("vim", help="Get Vim Helper")
def vim():
    vim_helper.show()


@tools_cli.command("bash", help="Get Bash Helper")
def bash():
    bash_helper.show()


@tools_cli.command("jupyter", help="Get Jupyter Helper")
def jupyter():
    jupyter_helper.show()


@tools_cli.command("gray2rgb", help="Conver Gray Image to RGB")
@click.option("--src_path", "-s", required=True, help="src gray image file path")
@click.option("--dest_path", "-d", default="./", help="dest RGB image file path")
@click.option("--h_num", "-h", default=1, help="h num for parallel execute")
@click.option("--w_num", "-w", default=1, help="w num for parallel execute")
def gray2rgb(src_path, dest_path, h_num, w_num):
    if h_num > 1 or w_num > 1:
        eut.convertGray2RGB_Muti(src_path, dest_path=dest_path, h_num=h_num, w_num=w_num)
    else:
        eut.convertGray2RGB(src_path, dest_path=dest_path)


@tools_cli.command("gray2rgb", help="Conver RGB Image to Gray")
@click.option("--src_path", "-s", required=True, help="src RGB image file path")
@click.option("--dest_path", "-d", default="./", help="dest gray image file path")
@click.option("--h_num", "-h", default=1, help="h num for parallel execute")
@click.option("--w_num", "-w", default=1, help="w num for parallel execute")
def rgb2gray(src_path, dest_path, h_num, w_num):
    if h_num > 1 or w_num > 1:
        eut.convertRGB2Gray_Muti(src_path, dest_path=dest_path, h_num=h_num, w_num=w_num)
    else:
        eut.convertRGB2Gray(src_path, dest_path=dest_path)


@tools_cli.command("loadnpy", help="load npy from file and print npy datas")
@click.option("--src_path", "-s", required=True, help="npy file path")
def loadnpy(src_path):
    eut.load_npy(src_path)


if __name__ == '__main__':
    tools_cli()



import torch

from erhsh.common.checkpoint import CheckpointLoader


class PtPthLoader(CheckpointLoader):
    def _load_checkpoint(self):
        param_dict = torch.load(self.checkpoint_path)
        return {k: v.detach().numpy() for k, v in param_dict.items()}



import mindspore
import numpy as np
from mindspore import context, ms_function
from mindspore import nn, Tensor
from mindspore.ops import GradOperation, operations as P

context.set_context(device_target="CPU")
context.set_context(mode=context.GRAPH_MODE)


class OpNetWrapper(nn.Cell):
    def __init__(self, op, *args, **kwargs):
        super(OpNetWrapper, self).__init__()
        self.op = op
        self.args = args
        self.kwargs = kwargs

    def construct(self, *inputs):
        return self.op(*inputs, *self.args, **self.kwargs)


class Grad(nn.Cell):
    def __init__(self, network):
        super(Grad, self).__init__()
        self.grad = GradOperation(get_all=True, sens_param=True)
        self.network = network

    @ms_function
    def construct(self, input_, output_grad):
        return self.grad(self.network)(input_, output_grad)


########################################################################################################################
# Test Code
########################################################################################################################

def test_00_base():
    op = P.Abs()
    op_wrapper = OpNetWrapper(op)

    input_x = Tensor(np.array([-1, 0, 1]), mindspore.float32)
    outputs = op_wrapper(input_x)

    print("input is:", input_x)
    print("output is:", outputs)
    assert np.allclose(outputs.asnumpy(), [1., 0., 1.])


def test_01_grad():
    op = P.Abs()
    op_wrapper = OpNetWrapper(op)

    input_x = Tensor(np.array([-3, -2, -1, -0, 0, 1, 2, 3]), mindspore.float32)
    outputs = op_wrapper(input_x)

    sens = Tensor(np.arange(int(np.prod(outputs.shape))).reshape(outputs.shape), mindspore.float32)
    backward_net = Grad(op_wrapper)
    input_grad = backward_net(input_x, sens)

    print("input is:", input_x)
    print("outputs is:", outputs)
    print("sens is:", sens.asnumpy())
    print("input_grad is:", input_grad[0].asnumpy())
    assert np.allclose(input_grad[0].asnumpy(), [-0., -1., -2., 0., 0., 5., 6., 7.])


if __name__ == '__main__':
    test_00_base()
    test_01_grad()



from PIL import Image, ImageFile
import numpy as np
import os

ImageFile.LOAD_TRUNCATED_IMAGES=True
Image.MAX_IMAGE_PIXELS=None

PIXEL_TO_RGB = {
    0: [0, 0, 0],   # blank
    1: [255, 0, 0], # red           --> bare-land 
    2: [0, 255, 0], # green         
    3: [0, 0, 255], # blue          --> vegetation
    4: [255, 255, 0], # yellow
    5: [255, 0, 255], # pink        --> building
    6: [0, 255, 255], # cyan        --> road
    7: [127, 0, 0], # dark red
    8: [0, 127, 0], # dark green
    9: [0, 0, 127], # dark blue
    10: [127, 127, 0], # dark yellow --> warter-body
    11: [127, 0, 127], # dark pink
    12: [0, 127, 127], # dark cyan
}

RGB_TO_PIXEL = {"_".join([str(v) for v in value]): key for key, value in PIXEL_TO_RGB.items()}


def gray2RGB(img):
    H, W = img.shape
    print(">>>>>>>>>>", img)
    img_rgb = np.random.randint(0, 256, size=[H, W, 3], dtype=np.uint8)
    for i in range(H):
        for j in range(W):
            k = img[i][j]
            rgb = PIXEL_TO_RGB.get(k, [255, 255, 255])
            img_rgb[i][j] = rgb
    return img_rgb


def RGB2gray(img_rgb):
    H, W, C = img_rgb.shape
    img = np.zeros((H, W), dtype=np.uint8)
    for i in range(H):
        for j in range(W):
            rgb = img_rgb[i][j]
            rgb = "_".join([str(v) for v in rgb])
            k = RGB_TO_PIXEL.get(rgb, 255)
            img[i][j] = k
    return img


def convertGray2RGB(img_path, dest_path=None):
    print(">>> img_path={}, dest_path={}".format(img_path, dest_path))
    img_data = Image.open(img_path).convert("L")
    img_np = np.array(img_data)
 
    print(">>> gray2RGB before, img_np.shape={}".format(img_np.shape))
    img_np = gray2RGB(img_np)
    print(">>> gray2RGB after, img_np.shape={}".format(img_np.shape))

    im = Image.fromarray(img_np)

    if dest_path is None:
        name = os.path.splitext(img_path)[0]
        ext = os.path.splitext(img_path)[1]
        dest_path = "{}_rgb{}".format(name, ext)
    elif os.path.isdir(dest_path):
        basename = os.path.basename(img_path)
        name = os.path.splitext(basename)[0]
        ext = os.path.splitext(basename)[1]
        dest_path = os.path.join(dest_path, "{}_rgb{}".format(name, ext))

    print(">>> save to dest_path={} start...".format(dest_path))
    im.save(dest_path)
    print(">>> save to dest_path={} success.".format(dest_path))


def convertGray2RGB_Muti(img_path, dest_path=None, h_num=1, w_num=1):
    print(">>> img_path={}, dest_path={}".format(img_path, dest_path))
    img_data = Image.open(img_path).convert("L")
    img_np = np.array(img_data)
 
    from erhsh import utils as eut

    processor = eut.MutiProcessor(img_np, h_num, w_num, cube_func=gray2RGB)
    ret = processor.process()

    if dest_path is None:
        name = os.path.splitext(img_path)[0]
        ext = os.path.splitext(img_path)[1]
        dest_path = "{}_rgb{}".format(name, ext)
    elif os.path.isdir(dest_path):
        basename = os.path.basename(img_path)
        name = os.path.splitext(basename)[0]
        ext = os.path.splitext(basename)[1]
        dest_path = os.path.join(dest_path, "{}_rgb{}".format(name, ext))

    to_image = Image.new("RGB", img_np.shape[:2][::-1])
    for k, v in ret.items():
        h_s, w_s, _, _ = tuple([int(x) for x in k.split("_")])
        to_image.paste(Image.fromarray(v), (w_s, h_s))

    to_image.save(dest_path)


def convertRGB2Gray(img_path, dest_path=None):
    print(">>> img_path={}, dest_path={}".format(img_path, dest_path))
    img_data = Image.open(img_path)
    img_rgb = np.array(img_data)
 
    print(">>> RGB2gray before, img_rgb.shape={}".format(img_rgb.shape))
    img_gray = RGB2gray(img_rgb)
    print(">>> RGB2gray after, img_gray.shape={}".format(img_gray.shape))

    img_gray = Image.fromarray(img_gray)

    if dest_path is None:
        name = os.path.splitext(img_path)[0]
        ext = os.path.splitext(img_path)[1]
        dest_path = "{}_gray{}".format(name, ext)
    elif os.path.isdir(dest_path):
        basename = os.path.basename(img_path)
        name = os.path.splitext(basename)[0]
        ext = os.path.splitext(basename)[1]
        dest_path = os.path.join(dest_path, "{}_gray{}".format(name, ext))

    print(">>> save to dest_path={} start...".format(dest_path))
    img_gray.save(dest_path)
    print(">>> save to dest_path={} success.".format(dest_path))


def convertRGB2Gray_Muti(img_path, dest_path=None, h_num=1, w_num=1):
    print(">>> img_path={}, dest_path={}".format(img_path, dest_path))
    img_data = Image.open(img_path)
    img_rgb = np.array(img_data)
 
    from erhsh import utils as eut

    processor = eut.MutiProcessor(img_rgb, h_num, w_num, cube_func=RGB2gray)
    ret = processor.process()

    if dest_path is None:
        name = os.path.splitext(img_path)[0]
        ext = os.path.splitext(img_path)[1]
        dest_path = "{}_gray{}".format(name, ext)
    elif os.path.isdir(dest_path):
        basename = os.path.basename(img_path)
        name = os.path.splitext(basename)[0]
        ext = os.path.splitext(basename)[1]
        dest_path = os.path.join(dest_path, "{}_gray{}".format(name, ext))

    to_image = Image.new("L", img_rgb.shape[:2][::-1])
    for k, v in ret.items():
        h_s, w_s, _, _ = tuple([int(x) for x in k.split("_")])
        to_image.paste(Image.fromarray(v), (w_s, h_s))

    to_image.save(dest_path)


if __name__ == '__main__':
    mock_gray_img_path = "tmp_gray.png"
    mock_rgb_img_path = "tmp_rgb.png"
    mock_gray_img_np = np.random.randint(1, 10, (100, 100))
    mock_gray_img = Image.fromarray(mock_gray_img_np)
    mock_gray_img.save(mock_gray_img_path)
    convertGray2RGB(mock_gray_img_path, dest_path=mock_rgb_img_path)
    convertRGB2Gray(mock_rgb_img_path, dest_path=mock_gray_img_path)
    mock_gray_img_np2 = np.array(Image.open(mock_gray_img_path).convert("L"))
    assert np.all(mock_gray_img_np == mock_gray_img_np2)
    os.remove(mock_gray_img_path)
    os.remove(mock_rgb_img_path)


import click

import erhsh
from erhsh.pt.demo import demo_mgmt


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo("Supported PyTorch Demo version is %s" % erhsh.pt.demo.version.__version__)
    ctx.exit()


@click.group(help='This is PyTorch Demo Command Line Console.')
@click.option("--version", is_flag=True, callback=print_version, expose_value=False,
              help="Show Supported PyTorch Demo version.")
def ptd_cli():
    pass


@ptd_cli.command("ops", help="PyTorch Operator Fetcher")
@click.argument("name", type=str, default="")
@click.option("--fetch_to", "-ft", type=str, help="fetch to DIR.")
@click.option("--view", "-v", is_flag=True, default=False, help="view in screen")
def ops(name, fetch_to, view):
    mgmt = demo_mgmt.DemoMgmt(name="ops", dir_name="operator")
    if name:
        mgmt.fetch_demo(name, fetch_to, view)
    else:
        mgmt.print_demo_list()


if __name__ == '__main__':
    ptd_cli()



from collections import OrderedDict

import numpy as np

from erhsh.utils import create_dir


class DatasetWrapper(object):
    def __init__(self, ds):
        from mindspore.dataset.engine import BatchDataset, RepeatDataset
        assert isinstance(ds, (BatchDataset, RepeatDataset))
        self.ds = ds
        self.odict = OrderedDict()
        self.__parse_ds()

    def __parse_ds(self):
        for x in self.ds.create_dict_iterator():
            for k in x.keys():
                v = np.array([x.get(k).asnumpy()])
                if k not in self.odict:
                    self.odict[k] = v
                else:
                    exist_v = self.odict[k]
                    self.odict[k] = np.vstack((exist_v, v))

    def __getitem__(self, item):
        ret = self.odict.get(item)
        if ret is not None:
            return ret

        if isinstance(item, int):
            return self.odict.get(list(self.odict.keys())[item])

        return None

    def ds_info(self, prefix="|-"):
        print("{}get_dataset_size={}".format(prefix, self.ds.get_dataset_size()), flush=True)
        print("{}get_repeate_count={}".format(prefix, self.ds.get_repeat_count()), flush=True)
        print("{}output_types={}".format(prefix, self.ds.output_types()), flush=True)
        print("{}output_shapes={}".format(prefix, self.ds.output_shapes()), flush=True)

        for x in self.ds.create_dict_iterator():
            keys = x.keys()
            for k in keys:
                v = x.get(k)
                print("{} {} -> {} {}".format(prefix, k, v.shape, v.dtype), flush=True)
            break


# store images
CACHE_DIR = '.cache_ds'


class ImageDatasetWrapper(DatasetWrapper):
    def __init__(self, ds, img_type="CHW"):
        super().__init__(ds)
        self.img_store_dir = CACHE_DIR
        self.img_type = img_type

    def save_as_img(self, img_key=None, img_store_dir=None):
        if img_store_dir is not None:
            self.img_store_dir = img_store_dir

        xnchw = self.__getitem__(img_key)
        if xnchw is None:
            raise RuntimeError("Key not found : " + img_key)

        self.__conv2pic(xnchw)

    def __getitem__(self, item):
        return super(ImageDatasetWrapper, self).__getitem__(item)

    def __conv2pic(self, xnchw):
        assert xnchw.ndim == 5
        img_store_dir = create_dir(self.img_store_dir)
        for i, nchw in enumerate(xnchw):
            batch_store_dir = create_dir('{}/batch-{}'.format(img_store_dir, i))
            for j, chw in enumerate(nchw):
                image_name = '{}/{}_{}.jpg'.format(batch_store_dir, j, '-'.join((str(x) for x in chw.shape)))
                self.__conv2pic0(chw, image_name)

    def __conv2pic0(self, chw, image_name):
        assert chw.ndim == 3
        if self.img_type == "CHW":
            hwc = np.transpose(chw, (2, 1, 0))
            if hwc.dtype == np.int32:
                hwc = hwc.astype(np.uint8)
        else:
            hwc = chw

        import matplotlib.pyplot as plt
        plt.imsave(image_name, hwc)



from . import demo, ds, tools, version
from .demo import *
from .ds import *
from .tools import *
from .train import *

__all__ = []
__all__.extend(demo.__all__)
__all__.extend(ds.__all__)
__all__.extend(tools.__all__)
__all__.extend(train.__all__)



import os

import erhsh.ms as ems


def create_dataset(data_path, is_train=True, batch_size=32):
    # import
    import mindspore.common.dtype as mstype
    import mindspore.dataset.engine as de
    import mindspore.dataset.transforms.c_transforms as C2
    import mindspore.dataset.vision.c_transforms as C
    from mindspore.common import set_seed
    from mindspore.dataset.vision import Inter

    set_seed(1)

    # shard
    num_shards = shard_id = None
    rand_size = os.getenv("RANK_SIZE")
    rand_id = os.getenv("RANK_ID")
    if rand_size is not None and rand_id is not None:
        num_shards = int(rand_size)
        shard_id = int(rand_id)

    # define dataset
    data_path = os.path.join(data_path, "train" if is_train else "test")
    ds = de.MnistDataset(data_path, num_shards=num_shards, shard_id=shard_id)

    # define ops
    comps_ops = list()
    comps_ops.append(C.Resize((32, 32), interpolation=Inter.LINEAR))
    comps_ops.append(C.Rescale(1 / 0.3081, -1 * 0.1307 / 0.3081))
    comps_ops.append(C.Rescale(1 / 255., 0.))
    comps_ops.append(C.HWC2CHW())

    # map ops
    ds = ds.map(input_columns=["image"], operations=comps_ops, num_parallel_workers=8)
    ds = ds.map(input_columns=["label"], operations=C2.TypeCast(mstype.int32), num_parallel_workers=8)

    # batch & repeat
    ds = ds.shuffle(buffer_size=1000)
    ds = ds.batch(batch_size=batch_size, drop_remainder=is_train)
    ds = ds.repeat(count=1)

    return ds


if __name__ == '__main__':
    dataset = create_dataset("D:/Data/dataset/mnist", is_train=True)
    ems.print_ds_info(dataset)
    ems.print_ds_data(dataset)



import inspect

import numpy as np


class DatasetMocker(object):
    def __init__(self, data_len=256, batch_size=32, repeat_count=1, features=None, feature_names=None, npw=8):
        self.__data_len = data_len
        self.__batch_size = batch_size
        self.__repeat_count = repeat_count
        self.__features = features
        self.__feature_names = feature_names
        self.__npw = npw

    def set_data_len(self, data_len):
        self.__data_len = data_len
        return self

    def set_batch_size(self, batch_size):
        self.__batch_size = batch_size
        return self

    def set_repeat_count(self, repeat_count):
        self.__repeat_count = repeat_count
        return self

    def set_features(self, features):
        self.__features = features
        return self

    def set_feature_names(self, feature_names):
        self.__feature_names = feature_names
        return self

    def set_npw(self, npw):
        self.__npw = npw
        return self

    def mock(self):
        if self.__features is None:
            self.__features = {
                'image': lambda row_idx: np.ones((3, 100, 100), dtype=np.int32) * row_idx % 256,
                'label': lambda row_idx: np.array(np.ones((), dtype=np.int32) * row_idx % 256)
            }

        if self.__feature_names is None:
            if isinstance(self.__features, dict):
                self.__feature_names = list(self.__features.keys())
            elif isinstance(self.__features, (list, tuple)):
                self.__feature_names = ['feature_' + str(i) for i, _ in enumerate(self.__features)]
            else:
                self.__feature_names = ['undefined']

        def ds_gen_func():
            for row_idx in range(self.__data_len):
                if isinstance(self.__features, (dict, list, tuple)):
                    feature_values = self.__features
                    if isinstance(self.__features, dict):
                        feature_values = self.__features.values()
                    yield tuple(
                        v if not callable(v) else v(row_idx) if len(inspect.getfullargspec(v).args) == 1 else v()
                        for v in feature_values)
                else:
                    yield tuple([self.__features])

        import mindspore.dataset as de
        dataset = de.GeneratorDataset(ds_gen_func, column_names=self.__feature_names, num_parallel_workers=self.__npw)
        dataset = dataset.batch(batch_size=self.__batch_size, drop_remainder=True)
        dataset = dataset.repeat(self.__repeat_count)

        return dataset



import numpy as np

def load_npy(npy_path):
    print(">>> npy path:", npy_path)
    np_data = np.load(npy_path)
    print(">>> npy shape:", np_data.shape)
    print(">>> npy data: \n", np_data)



if __name__ == "__main__":
    npy_path="/home/cj/dump_sync/rank_0/FCN8s/1/0/Conv2DBackpropInput.Conv2DTranspose-op251.0.0.1633703088299848.input.1.DefaultFormat.npy"
    load_npy(npy_path=npy_path)
    pass


import json
import os

DEVICE_NUM = 8
HCCN_CONF_FILE = '/etc/hccn.conf'

SERVER_ID = os.getenv('SSH_CONNECTION', 'ip port x.x.x.x').split()[2]


def gen_rank_table_file(server_id=None, visible_devices=None, hccn_conf_file=None):
    hccn_conf_file = HCCN_CONF_FILE if hccn_conf_file is None else hccn_conf_file

    def get_device_ips(cfg_file):
        rst = {}
        with open(cfg_file, 'r') as f:
            print("hccn_conf_file:", cfg_file)
            for line in f.readlines():
                if not line.startswith('address_'):
                    continue
                device_id, device_ip = line.strip().split('=')
                device_id = device_id.split('_')[1]
                rst[device_id] = device_ip
                print('device_id:{}, device_ip:{}'.format(device_id, device_ip))
        return rst

    if os.path.exists(hccn_conf_file):
        device_ips = get_device_ips(hccn_conf_file)
    else:
        print("Warning: Please specify the param `hccn_conf_file` so that we can get the right device ip.")
        device_ips = {
            '0': '192.168.101.101',
            '1': '192.168.102.101',
            '2': '192.168.103.101',
            '3': '192.168.104.101',
            '4': '192.168.101.102',
            '5': '192.168.102.102',
            '6': '192.168.103.102',
            '7': '192.168.104.102',
        }

    visible_devices = [str(x) for x in range(DEVICE_NUM)] if visible_devices is None else visible_devices
    visible_devices = visible_devices.split(',') if isinstance(visible_devices, str) else visible_devices
    device_num = len(visible_devices)

    # rank_table template
    rank_table = {
        'version': '1.0',
        'server_count': '1',
        'server_list': [],  # set later
        'status': 'completed'
    }
    server_list = [
        {
            'server_id': str(SERVER_ID if server_id is None else server_id),
            'device': [],  # set later
            'host_nic_ip': 'reserve'
        }
    ]
    rank_table['server_list'] = server_list

    device = []
    for i in range(device_num):
        instance = {
            'rank_id': str(i),
            'device_id': visible_devices[i],
            'device_ip': device_ips[visible_devices[i]],
        }
        device.append(instance)
    server_list[0]['device'] = device

    rank_table_file = os.path.join(os.getcwd(), 'rank_table_{}p_{}.json'.format(device_num, '-'.join(visible_devices)))
    with open(rank_table_file, 'w') as table_fp:
        json.dump(rank_table, table_fp, indent=4)

    soft_link_file = 'rank_table_{}p.json'.format(device_num)
    if os.path.exists(soft_link_file):
        os.remove(soft_link_file)
    os.symlink(rank_table_file, soft_link_file)

    return rank_table_file


if __name__ == '__main__':
    gen_rank_table_file()



import torch


def test_abs():
    # inputs
    input_x = torch.tensor([1, -1, 0, 2, -5], dtype=torch.float32, requires_grad=True)
    print("input_x:", type(input_x), input_x.detach().numpy())

    # forward
    outputs = torch.abs(input_x)
    print("outputs:", type(outputs), outputs.detach().numpy())

    # bakward
    ones = torch.ones(size=(5,)) * 10
    outputs.backward(gradient=ones)
    grad = input_x.grad
    print("input_x.grad:", type(grad), grad.numpy())


if __name__ == '__main__':
    test_abs()



from .file_util import *
from .print_util import *
from .safe_convert import *
from .img_util import *
from .img_comp_split import *
from .calc_iou import *
from .logger_util import *
from .mutiprocessing_util import *
from .npy_loader import *

__all__ = [
    "list_sub_dirs",
    "flatten_dir_path",
    "create_dir",
    "safe2int",
    "safe2float",
    "TblPrinter",
    "MutiProcessor",
    "get_logger",
    "gray2RGB",
    "convertGray2RGB",
    "convertGray2RGB_Muti",
    "convertRGB2Gray",
    "convertRGB2Gray_Muti",
    "calc_mean_iou",
    "img_compose",
    "load_npy",
]



import click

import erhsh
from erhsh.pt.tools.pth import PtPthLoader


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo("Supported PyTorch version is %s" % erhsh.pt.version.__version__)
    ctx.exit()


@click.group(help='This is PyTorch Command Line Console.')
@click.option("--version", is_flag=True, callback=print_version, expose_value=False,
              help="Show Supported PyTorch version.")
def pt_cli():
    pass


@pt_cli.command("pth", help="Get pth information")
@click.option("--pth_path", "-p", required=True, help="checkpoint file path")
@click.option("--filter_key", "-f", default=None, help="key filter condition, regular supported")
@click.option("--key", "-k", default=None, help="specify key")
@click.option("--dump", "-d", is_flag=True, default=False, help="flag dump data")
@click.option("--dump_to", "-t", default="./", help="where dump to")
def pth(pth_path, filter_key, key, dump, dump_to):
    pth_loader = PtPthLoader(pth_path)
    if key:
        if dump:
            pth_loader.get_dump(key, dump_to=dump_to)
        else:
            pth_loader.get(key)
    else:
        if dump:
            pth_loader.list_dump(filter_key=filter_key, dump_to=dump_to)
        else:
            pth_loader.list(filter_key=filter_key)


if __name__ == '__main__':
    pt_cli()



import json
import os
import shutil
import sys

import erhsh.utils as eut

DEFAULT_FETCH_TO = "emsd"


class DemoMgmt:
    def __init__(self, name=None, dir_name="demo"):
        self.__name = name
        self.__mgmt_path = self.__get_mgmt_path()
        self.__demo_dir_path = os.path.join(self.__mgmt_path, dir_name)

    def __get_mgmt_path(self):
        return os.path.dirname(os.path.realpath(sys.modules[self.__module__].__file__))

    def __demo_list(self):
        rst = [f for f in os.listdir(self.__demo_dir_path) if
               os.path.isfile(os.path.join(self.__demo_dir_path, f)) and '__init__.py' != f]
        rst.sort()
        return rst

    def print_demo_list(self):
        tp = eut.TblPrinter("ID", "File Name")
        for i, fn in enumerate(self.__demo_list()):
            tp.add_row(str(i), fn)
        tp.print()

    def __find_demo(self, key):
        fs = self.__demo_list()

        fid = eut.safe2int(key)
        if isinstance(fid, int) and fid < len(fs):
            return fs[fid]

        if key in fs:
            return key

        return None

    def __gen_jupyter_file(self, dest_file):
        with open(os.path.join(self.__mgmt_path, 'jupyter_template.ipynb'), mode='r') as f:
            data = json.load(f)
        data['cells'][0]['source'].append('%load ' + os.path.basename(dest_file))

        jupyter_dest_file = dest_file.replace('.py', '.ipynb')
        with open(jupyter_dest_file, mode='w') as f:
            json.dump(data, f, indent=4)

        return jupyter_dest_file

    def fetch_demo(self, key, fetch_to=None, view=False):
        os.system("")
        file_name = self.__find_demo(key)
        if file_name is None:
            print(">>> Please use The `ID' or 'File Name' from table blow!")
            self.print_demo_list()
            return
        src_file = os.path.join(self.__demo_dir_path, file_name)

        if view:
            file_gen = (row for row in open(src_file))
            for line in file_gen:
                print("\033[1;92m" + line + "\033[0m", end="")
            return

        if fetch_to is None:
            fetch_to = '_'.join([DEFAULT_FETCH_TO, self.__name]) if DEFAULT_FETCH_TO else DEFAULT_FETCH_TO
        fetch_to = fetch_to if fetch_to.startswith("/") or ":" in fetch_to else os.path.join(os.getcwd(), fetch_to)
        os.makedirs(fetch_to, exist_ok=True)

        dest_file = os.path.join(fetch_to, file_name)
        if os.path.exists(dest_file):
            print("\033[33m************* Warning: File Exist. Override It! ************\033[0m")
        shutil.copy(src_file, dest_file)
        print("\033[1;37;104mCopy To:\033[0m \033[1;92m{}\033[0m".format(dest_file))

        if dest_file.endswith('.py'):
            jupyter_dest_file = self.__gen_jupyter_file(dest_file)
            print("\033[1;37;104mGenerate .ipynb:\033[0m \033[1;92m{}\033[0m".format(jupyter_dest_file))


ds_mgmt = DemoMgmt(name="ds", dir_name="dataset")
net_mgmt = DemoMgmt(name="net", dir_name="network")
ops_mgmt = DemoMgmt(name="ops", dir_name="operator")
res_mgmt = DemoMgmt(name="res", dir_name="resource")
train_mgmt = DemoMgmt(name="train", dir_name="training")

if __name__ == '__main__':
    ds_mgmt.print_demo_list()
    net_mgmt.print_demo_list()
    ops_mgmt.print_demo_list()
    res_mgmt.print_demo_list()
    train_mgmt.print_demo_list()



import time


def get_time_monitor():
    from mindspore.train.callback import Callback

    class _TimeMonitor(Callback):
        def __init__(self):
            super(_TimeMonitor, self).__init__()
            self.epoch_start_time = 0
            self.epoch_end_time = 0
            self.epoch_start_step_num = 0
            self.epoch_end_step_num = 0

        def epoch_begin(self, run_context):
            self.epoch_start_time = time.time()

            cb_params = run_context.original_args()
            self.epoch_start_step_num = cb_params['cur_step_num']

        def epoch_end(self, run_context):
            self.epoch_end_time = time.time()

            cb_params = run_context.original_args()
            self.epoch_end_step_num = cb_params['cur_step_num']

            epoch_num = cb_params['epoch_num']
            cur_epoch_num = cb_params['cur_epoch_num']

            epoch_time_cost = (self.epoch_end_time - self.epoch_start_time) * 1000
            epoch_step_num = self.epoch_end_step_num - self.epoch_start_step_num
            per_step_cost = epoch_time_cost / epoch_step_num

            msg = "Epoch[{:2d}/{:2d}], Step[{:2d}], epoch time: {:5.3f}ms, per step time: {:5.3f}ms". \
                format(cur_epoch_num, epoch_num, self.epoch_end_step_num, epoch_time_cost, per_step_cost)

            print(msg, flush=True)

    return _TimeMonitor()


TimeMonitor = get_time_monitor



import os

import erhsh.ms as ems

parser = ems.ArgumentParser()
args, _ = parser.parse_known_args()


# # train test config
# args.data_path = "./mnist"
# args.batch_size = 32
# args.device_target = "Ascend"
# args.graph_mode = True
# args.epoch_size = 10
#
# # eval test config
# args.ckpt_path = "./ckpt/lenet-10_1875.ckpt"
# args.do_eval = True

# print("parser:", parser)
# print("args:", args)


def main():
    from mindspore import nn

    # dataset
    if os.path.exists(args.data_path):
        dataset = ems.create_mnist_dataset(args.data_path, is_train=(not parser.do_eval), batch_size=args.batch_size)
    else:
        dataset = ems.create_mocker_dataset()
    ems.print_ds_info(dataset)

    # network
    network = ems.LeNet5()

    # loss & opt
    loss_fn = nn.SoftmaxCrossEntropyWithLogits(sparse=True, reduction="mean")
    optimizer = nn.Momentum(network.trainable_params(), 0.01, 0.9)

    # train
    ems.Trainer('lenet', parser) \
        .set_dataset(dataset) \
        .set_network(network) \
        .set_loss_fn(loss_fn) \
        .set_optimizer(optimizer) \
        .set_callbacks([ems.TimeMonitor()]) \
        .run()


if __name__ == '__main__':
    main()



from erhsh.utils import TblPrinter


class BashHelper:
    def __init__(self):
        self.cmd_items = list()

    def add(self, *cmd_item):
        self.cmd_items.append(cmd_item)

    def show(self):
        tp = TblPrinter("Name", "Cmd Example")
        for cmd_item in self.cmd_items:
            tp.add_row(*cmd_item)
        tp.print()


bash_helper = BashHelper()

bash_helper.add("tr env", '''
ps -ef | grep python | grep -v grep | awk '{print $2}' | xargs -n 1 -I {} cat /proc/{}/environ | tr '\\0' '\\n' | grep DEV | sort | uniq
'''.strip())
bash_helper.add("ms commit id", '''
cat `pip show pip | grep Location | awk '{print $2}'`/mindspore/.commit_id
'''.strip())
bash_helper.add("pgrep", '''
pgrep -P 1 -t `who am i | awk '{print $2}'` -f python
'''.strip())



import os
import types
from importlib import import_module

from setuptools import setup, find_packages


def get_version():
    machinery = import_module("importlib.machinery")
    version_path = os.path.join(os.path.dirname(__file__), "erhsh", "version.py")
    module_name = "__erhshversion__"
    loader = machinery.SourceFileLoader(module_name, version_path)
    version_module = types.ModuleType(module_name)
    loader.exec_module(version_module)
    return version_module.__version__


if __name__ == '__main__':
    version = get_version()
    print("version:", version)

    setup(
        name="erhsh-python",
        version=version,
        description="This is a util python package",
        author="along",
        author_email="erhsh_165@126.com",
        packages=find_packages(),
        license="Apache 2.0",
        url="https://www.erhsh.com",
        entry_points={
            "console_scripts": [
                "ems=erhsh.ms.cli:ms_cli",
                "emsd=erhsh.ms.demo.cli:msd_cli",
                "ept=erhsh.pt.cli:pt_cli",
                "eptd=erhsh.pt.demo.cli:ptd_cli",
                "etools=erhsh.tools.cli:tools_cli",
            ]
        },
        include_package_data=True,
        install_requires=[
            "click",
        ],
        zip_safe=False,
    )



from erhsh.utils import TblPrinter


class GitHelper:
    def __init__(self):
        self.cmd_items = list()

    def add(self, *cmd_item):
        self.cmd_items.append(cmd_item)

    def show(self):
        tp = TblPrinter("Name", "Cmd Example")
        for cmd_item in self.cmd_items:
            tp.add_row(*cmd_item)
        tp.print()


git_helper = GitHelper()

git_helper.add('git lg', '''
    git config --global alias.lg "log --pretty=format:'%C(yellow)%h%Creset - [%Cgreen%ci %C(bold blue)%>(16,trunc)%cn%Creset] %<(80,trunc)%s %Cgreen(%cr) %C(cyan)%d%Creset'"
'''.strip())
git_helper.add('git blame <file>', 'git blame test.py -L 10')
git_helper.add('git log -- <file>', 'git log -- test.py')
git_helper.add('git log -p -- <file>', 'git log -p -2 --test.py')
git_helper.add('git sslVerify', 'git config --global http.sslVerify false')
git_helper.add('git postBuffer', 'git config --global http.postBuffer 1048576000')



import numpy as np


def create_net():
    from mindspore import nn, Parameter, Tensor

    class SimpleNet(nn.Cell):
        def __init__(self):
            super(SimpleNet, self).__init__()
            self.w = Parameter(Tensor(np.array([0.1], dtype=np.float32)), name="w")
            self.relu = nn.ReLU()

        def construct(self, *inputs, **kwargs):
            x, _ = inputs
            x = x + self.w
            x = self.relu(x)
            return x

    return SimpleNet()


if __name__ == '__main__':
    net = create_net()
    print(net)



from PIL import Image, ImageFile
import numpy as np
import os

ImageFile.LOAD_TRUNCATED_IMAGES=True
Image.MAX_IMAGE_PIXELS=None


def img_compose(image_names, H_NUM=None, W_NUM=None):
    if H_NUM is not None and W_NUM is not None:
        H_NUM = (0, H_NUM) if isinstance(H_NUM, int) else H_NUM
        W_NUM = (0, W_NUM) if isinstance(W_NUM, int) else W_NUM
        img_path_arr = np.array([[image_names.format(h, w) for w in range(*W_NUM)] for h in range(*H_NUM)])
        target_img_path = image_names.format("{}~{}".format(*H_NUM), "{}~{}".format(*W_NUM))
    elif H_NUM is None and W_NUM is not None:
        W_NUM = (0, W_NUM) if isinstance(W_NUM, int) else W_NUM
        img_path_arr = np.array([[image_names.format(w) for w in range(*W_NUM)]])
        target_img_path = image_names.format("{}~{}".format(*W_NUM))
    elif H_NUM is not None and W_NUM is None:
        H_NUM = (0, H_NUM) if isinstance(H_NUM, int) else H_NUM
        img_path_arr =np.array([[image_names.format(h)] for h in range(*H_NUM)])
        target_img_path = image_names.format("{}~{}".format(*H_NUM))
    else:
        img_path_arr = np.array(image_names)
        target_img_path = os.path.join(os.path.dirname(image_names[0][0]), "compose_{}x{}{}".format(*img_path_arr.shape[:2], os.path.splitext(image_names[0][0])[1]))
    print(">>>>>>>>>>>>>> target cube image list: \n", img_path_arr)
    print(">>>>>>>>>>>>>> compose image to: \n", target_img_path)
    # exit()

    H_NUM, W_NUM = img_path_arr.shape

    big_image_data = []
    cube_image_shape = None
    for h in range(H_NUM):
        for w in range(W_NUM):
            img_path = img_path_arr[h][w]
            # img_data = np.array(Image.open(img_path).convert("L"))
            img_data = np.array(Image.open(img_path))
            print(">>>>>", img_data.shape)
            cube_image_shape = img_data.shape
            big_image_data.append(img_data)
    big_image_data = np.array(big_image_data)
    big_image_data = big_image_data.reshape(H_NUM, W_NUM, *cube_image_shape)
    print("=========big image data shape:", big_image_data.shape)

    big_image_H = H_NUM * cube_image_shape[0]
    big_image_W = W_NUM * cube_image_shape[1]
    to_image = Image.new("RGB", (big_image_W, big_image_H))
    for h in range(H_NUM):
        h_s = h * cube_image_shape[0]
        for w in range(W_NUM):
            w_s = w * cube_image_shape[1]
            v = big_image_data[h][w]
            to_image.paste(Image.fromarray(v), (w_s, h_s))
    print(np.array(to_image).shape)
    to_image.save(target_img_path)



if __name__ == '__main__':
    names = """
        GF2_PMS1__L1A0000647768-MSS1
        GF2_PMS1__L1A0001094941-MSS1
        GF2_PMS1__L1A0001169100-MSS1
        GF2_PMS1__L1A0001491417-MSS1
        GF2_PMS1__L1A0001537716-MSS1
        GF2_PMS1__L1A0001553848-MSS1
        GF2_PMS1__L1A0001562063-MSS1
        GF2_PMS1__L1A0001734328-MSS1
        GF2_PMS2__L1A0000564692-MSS2
        GF2_PMS2__L1A0000647892-MSS2
        GF2_PMS2__L1A0000718813-MSS2
        GF2_PMS2__L1A0000948183-MSS2
        GF2_PMS2__L1A0001015596-MSS2
        GF2_PMS2__L1A0001028977-MSS2
        GF2_PMS2__L1A0001035795-MSS2
        GF2_PMS2__L1A0001119060-MSS2
        GF2_PMS2__L1A0001336975-MSS2
        GF2_PMS2__L1A0001378491-MSS2
        GF2_PMS2__L1A0001389317-MSS2
        GF2_PMS2__L1A0001416140-MSS2
        GF2_PMS2__L1A0001457157-MSS2
        GF2_PMS2__L1A0001537637-MSS2
        GF2_PMS2__L1A0001566653-MSS2
        GF2_PMS2__L1A0001577567-MSS2
        GF2_PMS2__L1A0001633212-MSS2
        GF2_PMS2__L1A0001708232-MSS2
        GF2_PMS2__L1A0001757317-MSS2
        GF2_PMS2__L1A0001757484-MSS2
        GF2_PMS2__L1A0001787080-MSS2
        GF2_PMS2__L1A0001886305-MSS2
    """

    names = names.split()
    base_dir = "/home/cj/pictmp.gid_gray/"

    img_paths = [os.path.join(base_dir, "{}_rank{}.tif".format(x, "{}")) for x in names]
    for p in img_paths:
        img_compose(p, W_NUM=32)

    lbl_paths = [os.path.join(base_dir, "{}_lbl_rank{}.tif".format(x, "{}")) for x in names]
    for p in lbl_paths:
        img_compose(p, W_NUM=32)


import os


template_txt = """
-- base dir
/home/cj/workspace

-- image matrix (2, 2, 4)
img_0_0.png img_0_1.png img_0_2.png img_0_3.png, img_1_0.png img_1_1.png img_1_2.png img_1_3.png 
img_2_0.png img_2_1.png img_2_2.png img_2_3.png, img_3_0.png img_3_1.png img_3_2.png img_3_3.png 

-- output image path
/home/cj/workspace/compose_img.png
"""


def gen_template():
    print(template_txt)


if __name__ == '__main__':
    gen_template()


import os
import sys

import erhsh.utils as eut
from erhsh.ms.tools.hccl_tool_v1 import gen_rank_table_file
from erhsh.ms.train._argparse import ArgumentParser


class Trainer(object):
    def __init__(self, name=None, parser: ArgumentParser = None):
        self.__name = 'net' if name is None else name
        self.__parser = ArgumentParser() if parser is None else parser
        self.__parser.parse_known_args()

        # train needed
        self.__dataset = None
        self.__network = None
        self.__loss_fn = None
        self.__optimizer = None
        self.__metrics = None
        self.__callbacks = None

        # train params
        self.__do_eval = self.__parser.do_eval
        self.__epoch_size = self.__parser.epoch_size
        self.__do_sink = self.__parser.do_sink
        self.__sink_size = self.__parser.sink_size
        self.__ckpt_path = self.__parser.ckpt_path
        self.__device_target = self.__parser.device_target
        self.__graph_mode = self.__parser.graph_mode
        self.__save_graphs = self.__parser.save_graphs
        self.__visible_devices = self.__parser.visible_devices

    # train needed setters
    def set_dataset(self, val):
        self.__dataset = val
        return self

    def set_network(self, val):
        self.__network = val
        return self

    def set_loss_fn(self, val):
        self.__loss_fn = val
        return self

    def set_optimizer(self, val):
        self.__optimizer = val
        return self

    def set_metrics(self, val):
        self.__metrics = val
        return self

    def set_callbacks(self, val):
        self.__callbacks = val
        return self

    # train params setters
    def set_do_eval(self, val):
        self.__do_eval = val
        return self

    def set_epoch_size(self, val):
        self.__epoch_size = val
        return self

    def set_do_sink(self, val):
        self.__do_sink = val
        return self

    def set_sink_size(self, val):
        self.__sink_size = val
        return self

    def set_ckpt_path(self, val):
        self.__ckpt_path = val
        return self

    def set_device_target(self, val):
        self.__device_target = val
        return self

    def set_graph_mode(self, val):
        self.__graph_mode = val
        return self

    def set_save_graphs(self, val):
        self.__save_graphs = val
        return self

    def set_visible_devices(self, val):
        self.__visible_devices = val
        return self

    def __update_cbs(self):
        if not isinstance(self.__callbacks, list):
            return

        from mindspore.train.callback import LossMonitor, ModelCheckpoint

        loss_cb_exist, ckpt_cb_exist = False, False
        for cb in self.__callbacks:
            loss_cb_exist = isinstance(cb, LossMonitor)
            ckpt_cb_exist = isinstance(cb, ModelCheckpoint)

        if not loss_cb_exist:
            self.__callbacks.append(LossMonitor())

        if not ckpt_cb_exist:
            self.__callbacks.append(ModelCheckpoint(prefix=self.__name, directory=self.__ckpt_path))

    def __train(self):
        # import
        from mindspore import Model, load_param_into_net, load_checkpoint

        # load params
        if self.__ckpt_path and os.path.isfile(self.__ckpt_path) and os.path.exists(self.__ckpt_path):
            load_param_into_net(self.__network, load_checkpoint(self.__ckpt_path))

        # loss_fn & optimizer & metrics
        model = Model(self.__network, loss_fn=self.__loss_fn, optimizer=self.__optimizer, metrics=self.__metrics)

        # train
        print(">>>>>>>>>>>>>>>>>>>>> train start ... <<<<<<<<<<<<<<<<<<<<<<")
        self.__update_cbs()
        model.train(self.__epoch_size, self.__dataset, callbacks=self.__callbacks,
                    dataset_sink_mode=self.__do_sink, sink_size=self.__sink_size)
        print(">>>>>>>>>>>>>>>>>>>>> train success ~ <<<<<<<<<<<<<<<<<<<<<<")

    def __eval(self):
        # import
        from mindspore import Model, load_param_into_net, load_checkpoint
        from mindspore.nn.metrics import Accuracy

        # load params
        if self.__ckpt_path:
            load_param_into_net(self.__network, load_checkpoint(self.__ckpt_path))
        else:
            print("Warning: `ckpt_path` is None, Please call func: `set_ckpt_path($ckpt_path)`.")
            return

        # loss_fn & optimizer & metrics
        model = Model(self.__network, loss_fn=self.__loss_fn, optimizer=self.__optimizer,
                      metrics={"Accuracy": Accuracy()} if self.__metrics is None else self.__metrics)

        # eval
        print(">>>>>>>>>>>>>>>>>>>>> eval start ... <<<<<<<<<<<<<<<<<<<<<<")
        result = model.eval(self.__dataset)
        print(">>>>>>>>>>>>>>>>>>>>> eval success ~ <<<<<<<<<<<<<<<<<<<<<<: result=", result)

    def __run_standalone(self):
        # import
        from mindspore import context
        from mindspore.communication import init
        from mindspore.context import ParallelMode

        # set context: device_target
        context.set_context(device_target=self.__device_target)

        # set context: mode
        if self.__graph_mode:
            context.set_context(mode=context.GRAPH_MODE)

        # set context: save_graphs
        context.set_context(save_graphs=self.__save_graphs)

        # set context: device_id
        device_id = int(os.environ.get("DEVICE_ID", 0))
        context.set_context(device_id=device_id)

        # init
        device_num = int(os.environ.get("DEVICE_NUM", 1))
        if device_num > 1 and "win32" not in sys.platform:
            context.reset_auto_parallel_context()
            context.set_auto_parallel_context(device_num=device_num, parallel_mode=ParallelMode.DATA_PARALLEL,
                                              gradients_mean=True)
            init()

        if self.__dataset is None:
            print("Warning: `dataset` is None. Please call func: `set_dataset($dataset)`.")

        if self.__network is None:
            print("Warning: `network` is None. Please call func: `set_network($network)`.")

        if self.__dataset is None or self.__network is None:
            return

        if self.__do_eval:
            self.__eval()
        else:
            self.__train()

    def __run_distribute(self, vds):
        train_name = '_'.join(['output', self.__name])
        net_output_dir = os.path.join(os.getcwd(), train_name)
        net_output_dir = eut.create_dir(net_output_dir)
        script_file = os.path.abspath(sys.argv[0])
        script_args = " ".join([x for x in sys.argv[1:] if "--visible_devices" not in x and "-vd" not in x])
        script_args = script_args + " no_distribute"

        rank_table_file = gen_rank_table_file(visible_devices=vds)
        rank_size = len(vds)
        log_file = None
        for rank_id in range(rank_size):
            device_id = vds[rank_id]
            cmd = list()
            cmd.append("export RANK_SIZE={}".format(rank_size))
            cmd.append("export RANK_ID={}".format(rank_id))
            cmd.append("export DEVICE_NUM={}".format(rank_size))
            cmd.append("export DEVICE_ID={}".format(device_id))
            cmd.append("export RANK_TABLE_FILE={}".format(rank_table_file))
            cmd.append("export MINDSPORE_HCCL_CONFIG_PATH={}".format(rank_table_file))

            device_run_dir = os.path.join(net_output_dir, "device{}".format(device_id))
            log_file = os.path.join(device_run_dir, "out.log")
            python_cmd = "python {} {} > {} 2>&1".format(script_file, script_args, log_file)

            cmd.append("rm -rf {}".format(device_run_dir))
            cmd.append("mkdir {}".format(device_run_dir))
            cmd.append("cd {}".format(device_run_dir))
            cmd.append("{} &".format(python_cmd))

            if "win32" in sys.platform:
                cmd = list()
                cmd.append("set RANK_SIZE={}".format(rank_size))
                cmd.append("set RANK_ID={}".format(rank_id))
                cmd.append("set DEVICE_NUM={}".format(rank_size))
                cmd.append("set DEVICE_ID={}".format(device_id))
                device_run_dir = device_run_dir.replace('/', '\\')
                if os.path.exists(device_run_dir):
                    cmd.append("rd /s /q {}".format(device_run_dir))
                cmd.append("mkdir {}".format(device_run_dir))
                cmd.append("cd {}".format(device_run_dir))
                cmd.append("start /b {}".format(python_cmd))

            run_cmd = " && ".join(cmd)
            # print("Run cmd: {}".format(run_cmd))
            os.system(run_cmd)
            print("Output log: {}".format(log_file))

        if log_file and os.path.exists(log_file):
            import threading
            thread = threading.Thread(target=lambda: os.system("tail -f {}".format(log_file)))
            thread.daemon = True
            thread.start()

    def run(self):
        self.__parser.print_args()
        if self.__visible_devices and 'no_distribute' not in sys.argv:
            vds = self.__visible_devices.split(",")
            self.__run_distribute(vds)
        else:
            self.__run_standalone()



class TblPrinter(object):
    def __init__(self, *header, tbl_name=None, col_separator=" | ", row_separator="-"):
        self._header = header
        self._col_num = len(header)
        self._col_widths = [len(h) for h in header]
        self._rows = []
        self._tbl_name = tbl_name
        self._col_separator = col_separator
        self._row_separator = row_separator

    def add_row(self, *row):
        if len(row) == self._col_num:
            for i in range(self._col_num):
                old_col_width = self._col_widths[i]
                new_col_width = len(row[i])
                if new_col_width > old_col_width:
                    self._col_widths[i] = new_col_width
        self._rows.append(row)
        return self

    def _print_header(self):
        self.__print_line()
        if self._tbl_name:
            print_info = ['{0:^{ilen}}'.format(self._tbl_name, ilen=self.__get_tbl_width())]
            print(self._col_separator.join(print_info))
            self.__print_line()

        print_info = ['{0:^{ilen}}'.format(self._header[i], ilen=ilen) for i, ilen in enumerate(self._col_widths)]
        print(self._col_separator.join(print_info))
        self.__print_line()

    def _print_body(self):
        for row in self._rows:
            if len(row) == self._col_num:
                print_info = ['{0:<{ilen}}'.format(row[i], ilen=ilen) for i, ilen in enumerate(self._col_widths)]
            else:
                print_info = ['{0:<{ilen}}'.format(str(col), ilen=len(str(col))) for i, col in enumerate(row)]
            print(self._col_separator.join(print_info))
        self.__print_line()

    def __print_line(self):
        print(self._row_separator * self.__get_tbl_width())

    def __get_tbl_width(self):
        return sum(self._col_widths) + len(self._col_separator)

    def print(self):
        self._print_header()
        self._print_body()
